#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "./tui.h"

const char* rainbow_color(size_t i) {
  switch (i % 5) {
  case 0: return FG_RED;
  case 1: return FG_YELLOW;
  case 2: return FG_GREEN;
  case 3: return FG_CYAN;
  case 4: return FG_BLUE;
  }
  return ""; /* cannot happen, just to silence the warning. */
}

void draw_rainbow_string(size_t x, size_t y, const char* s) {
  Size2 size = tui_size();
  for (size_t i = 0; *s != 0; ++i) {
    size_t x2 = (x + i) % size.x;
    tui_cell_at(x2, y)->content = *(s++);
    tui_cell_at(x2, y)->text_color = rainbow_color(i);
    tui_cell_at(x2, y)->background_color = BG_BLACK;
  }
}

int main(int argc, char** argv) {
  if (argc != 2) {
    printf("USAGE: rainbows TEXT\n");
    exit(1);
  }
  const char* text = argv[1];

  tui_init();

  size_t t = 0;

  while (1) {
    if (stdin_has_changed()) {
      char c = read_from_stdin();
      if (c == 'q')
        break;
    }

    Size2 size = tui_size();

    tui_clear();

    draw_rainbow_string(t, size.y / 2 - 1, text);
    draw_rainbow_string(t+1, size.y / 2, text);
    draw_rainbow_string(t+2, size.y / 2 + 1, text);

    tui_update();

    t += 1;

    usleep(100000);
  }

  tui_shutdown();

  return 0;
}
